# R Graphics with ggplot2
# Fall 2016
# UVa StatLab
# Clay Ford


# Helpful RStudio commands ------------------------------------------------

# Description       Windows & Linux       Mac 
# ----------------------------------------------------------------
# Run current line  Ctrl+Enter            Command+Enter
# Previous plot     Ctrl+Shift+PageUp     Command+Shift+PageUp 
# Next plot 	      Ctrl+Shift+PageDown 	Command+Shift+PageDown 


# Packages ----------------------------------------------------------------

# only submit these lines if you don't already have these packages installed. 
install.packages("ggplot2")
install.packages("gridExtra") # to fit multiple plots in a window
install.packages("dplyr") # for data manipulation

# We'll also use the scales package; when you install ggplot2, the scales
# package is also installed

library(ggplot2)
library(dplyr)

# Data --------------------------------------------------------------------

# Albemarle County real estate data.

# Downloaded from Office of Geographic Data Services, 19-Sept-2016
# http://www.albemarle.org/department.asp?department=gds&relpage=3914#Parcels

# Data for records with City identified as "CHARLOTTESVILLE", "EARLYSVILLE",
# "CROZET", "NORTH GARDEN","SCOTTSVILLE", "KESWICK"

# R script to download and prepare data:
# https://github.com/clayford/ggplot2/blob/master/albemarle_county_homes.R

# Read in data:
# homes <- read.csv("albemarle_real_estate.csv")
URL <- "http://people.virginia.edu/~jcf2d/workshops/ggplot2/albemarle_real_estate.csv"
homes <- read.csv(URL)

class(homes) # data.frame
str(homes) # view structure

# reorder the levels of Condition:
levels(homes$Condition) # out of order
homes$Condition <- factor(homes$Condition, 
                          levels = c("Substandard", "Poor", "Fair", 
                                     "Average", "Good", "Excellent"))

levels(homes$Condition) # better


# One Variable ------------------------------------------------------------

# The histogram helps us see how a continous variable is distributed
ggplot(homes, aes(x=FinSqFt)) + geom_histogram()

# note the message regarding "stat_bin".
# Recall "stat_bin" is the stat for geom_histogram.

# Says the documentation: "You should always override this value, exploring
# multiple widths to find the best to illustrate the stories in your data."

# Try a new binwidth or bins
ggplot(homes, aes(x=FinSqFt)) + geom_histogram(binwidth=50)
ggplot(homes, aes(x=FinSqFt)) + geom_histogram(bins = 40)
# try some others! Beware, setting small binwidths can tax your computer.

# what if we want a "True" Histogram with density instead of count?
ggplot(homes, aes(x=FinSqFt, y=..density..)) + 
  geom_histogram(binwidth=250)

# ..density.. is avariable generated by stat_bin; documented on the stat_bin
# page

# The reason we might want a "True" histogram is to overlay a density estimate:
ggplot(homes, aes(x=FinSqFt, y=..density..)) + 
  geom_histogram(binwidth=250, color="black", fill="white") +
  geom_density(color="red")
# Notice I "set" the color and fill values instead of "mapping" a variable


# Before we go further, important to note we can save a plot object and add to
# it:
p <- ggplot(homes, aes(x=FinSqFt))
p # nothing to see, no layers

# now add a geom layer
p + geom_histogram(binwidth=300)

# And now try geom_density(); notice I can set the aesthetic mapping in a geom
# function:
p + geom_density(aes(y = ..density..), color="red")

# and combine density and histogram
p + geom_histogram(aes(y = ..density..), bins = 75, alpha=1/3) + 
  geom_density(color="red") 
  
# alpha is an aesthetic that defines "transparancy"; range: 0 to 1

# facet by condition:
p + geom_histogram(bins=75) + facet_wrap(~ Condition)

# overlayed histograms by condition
ggplot(homes, aes(x=FinSqFt, fill=Condition)) + 
  geom_histogram(position = "identity", alpha=1/3, binwidth=250)
# position="identity" needed for overlapping; without they get stacked

# Try fewer categories
ggplot(subset(homes, Condition %in% c("Excellent","Good","Average")), 
       aes(x=FinSqFt, fill=Condition)) + 
  geom_histogram(position = "identity", alpha=1/3, binwidth=250) 



####################################
# YOUR TURN! 

# (1) Create a histogram of TotalValue, the total value of the home.
# Try some different bandwidths, say bins = 50 or binwidth = 50000:

# (2) Repeat #1, but now facet by City


####################################

# Bar Plots help us visualize how discrete values are distributed:
ggplot(homes, aes(x=Condition)) + geom_bar()

# the coord_flip() function allows us to flip the coordinate axis
ggplot(homes, aes(x=Condition)) + geom_bar() +
  coord_flip() 

# Let's map the Remodeled indicator to fill; 
# The result is weird since since Remodled is either 0 or 1
ggplot(homes, aes(x=Condition, fill=Remodeled)) + geom_bar()

# Let's declare Remodeled as a "factor";
ggplot(homes, aes(x=Condition, fill=factor(Remodeled))) + geom_bar()

# Note the "stacked" position. We can change that with the position argument:
ggplot(homes, aes(x=Condition, fill=factor(Remodeled))) + 
  geom_bar(position = "dodge")

# How can we fix the legend title?
# Recall that is a by-product of the fill scale. So we need to use a scale
# function.
ggplot(homes, aes(x=Condition, fill=factor(Remodeled))) + 
  geom_bar(position = "dodge") +
  scale_fill_discrete(name="Remodeled", labels = c("Original","Remodeled"))


# By default, the stat for geom_bar (stat_count) counts up things in your data 
# frame. It assumes you have one record per observation in your data. What if 
# you already have counts? Use stat ="identity" (stat_identity) and map value to
# the y aesthetic. 

# Example
dat <- data.frame(gender = c("male","female"), 
                  total = c(134, 162))
dat

# without stat="identity" we get an error 
ggplot(dat, aes(x=gender, y=total)) + geom_bar()

# with stat="identity"
ggplot(dat, aes(x=gender, y=total)) + geom_bar(stat="identity", width = 0.5)



####################################
# YOUR TURN! 

# (1) Create a barplot of counts for City

# (2) Repeat #1, but now map the fill aesthetic to Condition


####################################



# Two variables -----------------------------------------------------------

# The scatterplot allows you to visualize the relationship between two
# continuous variables.

# plot FinSqFt vs TotalValue:
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point()

# Lots of overplotting!
# What can we do about that? One solution is the alpha aesthetic. "alpha=1/10"
# means 10 points overplotted adds to a solid color
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point(alpha=1/10)

# Can also try smaller points using the shape aesthetic
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point(shape=".")

# maybe zoom in? We can do that with coord_cartesian()
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point(alpha=1/5) +
  coord_cartesian(xlim=c(0,5000),ylim=c(0,1e6))

# Another approach is to use facets: break the data into subsets 
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point() +
  facet_wrap(~ Condition)

# Maybe let the scales vary? 
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point() +
  facet_wrap(~ Condition, scales = "free")

# and we can combine our tactics
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/6) +
  facet_wrap(~ Condition, scales = "free")


# yet another strategy is to bin the points and count the number in each using
# geom_hex()
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_hex()
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_hex(bins = 50)



#################################### 
# YOUR TURN! Plot FinSqFt vs. YearBuilt. Put FinSqFt on the y axis. 
# Facet by Condition. Perhaps try an alpha setting?


####################################

# Recall this plot:
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/6) +
  facet_wrap(~ Condition)

# The y-axis scale would look better formatted as dollar amounts. The scales
# package can help with this. It has functions designed for this type of
# situation. When you install ggplot2, scales is installed as well.

# Three functions that are very handy: percent, comma, dollar;
# typically used in a scale function with the labels argument.

library(scales)
p2 <- ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/6) +
  facet_wrap(~ Condition) + 
  scale_y_continuous(labels=dollar) + 
  scale_x_continuous(labels=comma)
p2

# We can add a smooth line through our scatterplots with geom_smooth()
p2 + geom_smooth()
# warning due to Substandard only having only a few obs;
# let's just try fitting a straight linear regression line
p2 + geom_smooth(method="lm")

# turn off the confidence intervals
p2 + geom_smooth(method="lm", se=FALSE)

# We can add both smooth and straight lines
p2 + geom_smooth(se=FALSE, color="red") +
  geom_smooth(method="lm", se=FALSE)

# But notice ggplot2 does not create a legend for us identifying the lines!
# That's because the lines are created from statistical transformations of the
# data. There's nothing to map the color of the lines to in the data.
  
# However we can force a legend by defining a color aesthetic in each geom:
p2 + geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE)

# And then we can customize the legend if we like
p2 + geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE) +
  scale_color_manual("Fitted Lines", values = c("blue","red"))

# Let's just look at Average, Excellent and Good homes, and fit quadratic and
# cubic lines as well. We can replace the data frame using the %+% operator
p2 %+% 
  subset(homes, Condition %in% c("Average","Excellent","Good")) + 
  geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE) +
  geom_smooth(aes(color = "quadratic"), method="lm", formula = y ~ poly(x, 2), se=FALSE) +
  labs(color = "Fitted Line")


# Log Transformations in scatter plots

# Dollar amounts are often log transformed. It helps "squeeze together" the 
# large values and "spread out" the small values.

# before log transformation:
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point()

# Maybe try log base 10 transformation directly:
ggplot(homes, aes(x=log10(FinSqFt), y=log10(TotalValue))) + geom_point(alpha = 1/5)

# The scale of the axes is on the log10 scale because the data has been
# transformed.

# We can use scale functions to both transform the data and map the scale of the
# axes to the original data.
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/5) +
  scale_x_log10(labels=comma) + 
  scale_y_log10(labels=dollar) +
  geom_smooth(se=F) +
  geom_smooth(method="lm", se=F, color="red")


# Boxplots are good for visualizing a continous variable conditional on a
# discrete variable. Let's look at TotalValue by number of FullBaths:

ggplot(homes, aes(x=FullBath, y=TotalValue)) + geom_boxplot()

# oops. FullBath is not discrete. This works:
ggplot(homes, aes(x = FullBath, y = TotalValue, group = FullBath)) + geom_boxplot()

# and so does this (convert FullBath to factor):
ggplot(homes, aes(x=factor(FullBath), y=TotalValue)) + geom_boxplot()

p3 <- ggplot(homes, aes(x=factor(FullBath),y=TotalValue))
p3 + geom_boxplot()

p3 + geom_boxplot() + scale_y_continuous(labels=dollar)
# $9,000,000 homes with only two full baths?

# facet by Condition
p3 + geom_boxplot() + scale_y_continuous(labels=dollar) + 
  facet_wrap(~Condition) + labs(x = "Number of Full Baths")

# facet by City
p3 + geom_boxplot() + scale_y_continuous(labels=dollar) + 
  facet_wrap(~City) + labs(x = "Number of Full Baths")

# facet by City and Condition
p3 + geom_boxplot() + scale_y_continuous(labels=dollar) + 
  facet_grid(Condition ~ City) + labs(x = "Number of Full Baths")

# facet by City and Condition - filter out "Substandard" and "Poor"
p3 %+%
  subset(homes, Condition %in% c("Average","Excellent","Good")) + 
  geom_boxplot() + scale_y_continuous(labels=dollar) + 
  facet_grid(Condition ~ City) + labs(x = "Number of Full Baths")


# similar to boxplots: violin plots
ggplot(homes, aes(y = TotalValue, x = City)) + geom_violin()

# subset for homes with TotalValue < $1,000,000
ggplot(homes, aes(y = TotalValue, x = City)) + geom_violin() + ylim(0, 1e6)



####################################
# YOUR TURN! Make a boxplot (or violin plot) of FinSqFt by HalfBath. 



####################################

# A nice alternative to boxplots are stripcharts. Basically plot a 1d 
# scatterplot for each level of the discrete value. Works well for smaller data
# sets.

# 0 bedroom houses (?)
homes0 <- subset(homes, Bedroom==0)
nrow(homes0)

# plot total value per Number of Full Baths for homes with 0 bedrooms using 
# geom_jitter(). geom_jitter adds some width (left-right) "jitter" to the points
# so they don't overplot.
ggplot(homes0, aes(x=factor(FullBath), y=TotalValue)) + 
  geom_jitter(width = 0.1, height = 0) +
  scale_y_continuous(labels=dollar)

# Let's go back to the homes data...

# Plotting two "discrete" integer variables:
# Bedroom vs. FullBath
ggplot(homes, aes(x=Bedroom,y=FullBath)) + geom_point()

# geom_jitter() can help with this
ggplot(homes, aes(x=Bedroom,y=FullBath)) + geom_jitter()

# scales could be better; minor_breaks=NULL turns off the minor grid lines 
ggplot(homes, aes(x=Bedroom,y=FullBath)) + geom_jitter(alpha=1/5) +
  scale_x_continuous(breaks=0:16, minor_breaks=NULL) +
  scale_y_continuous(breaks=0:14, minor_breaks=NULL)


# line graphs are nice for connecting dots and showing a trend over time.

# plot number of houses built per year; 
# need to count up number of homes by YearBuilt
years <- homes %>% 
  group_by(YearBuilt) %>% 
  summarise(Freq = n())

str(years)

# now use geom_line()
ggplot(years, aes(x=YearBuilt, y=Freq)) + geom_line()

# again we can touch up the scales and labels
ggplot(years, aes(x=YearBuilt, y=Freq)) + geom_line() +
  scale_x_continuous(breaks=seq(1700,2000,50)) +
  labs(x="Year",y="Number of Homes")

# what year was the spike?
subset(years, Freq > 800)

# we can add the year to the plot using annotate()
ggplot(years, aes(x=YearBuilt, y=Freq)) + geom_line() +
  scale_x_continuous(breaks=seq(1700,2000,50)) +
  labs(x="Year",y="Number of Homes") +
  annotate("text", x = 1985, y = 800, label = "1973") 



# Plotting statistical summaries ------------------------------------------

# the stat_summary() function is useful for summarizing y values for unique x 
# values. The basic usage is to supply a function to the fun.y argument.

# Let's say I want to plot the mean TotalValue per YearBuilt:
ggplot(homes, aes(x=YearBuilt, y=TotalValue)) +
  stat_summary(fun.y="mean", colour="red", geom="point")

# we could also do it over a scatter plot of TotalValue vs YearBuilt
ggplot(homes, aes(x=YearBuilt, y=TotalValue)) + 
  geom_point() +
  stat_summary(fun.y="mean", colour="red", geom="point")

# zoom in on 1950 - 2015
ggplot(homes, aes(x=YearBuilt, y=TotalValue)) + 
  geom_point(shape=".") +
  stat_summary(fun.y="mean", colour="red", geom="point") +
  coord_cartesian(xlim=c(1950,2016), ylim=c(0,1e6))
# worth noting we're just zooming; the mean still calculated using values
# outside of view.

# let's jitter the points and make summery point bigger
ggplot(homes, aes(x=YearBuilt, y=TotalValue)) + 
  geom_point(position=position_jitter(w=0.2,h=0), shape=".") +
  stat_summary(fun.y="mean", colour="red", geom="point", size=3) +
  coord_cartesian(xlim=c(1950,2016), ylim=c(0,1e6))

# we can add the median and turn the points into lines
ggplot(homes, aes(x=YearBuilt, y=TotalValue)) + 
  geom_point(position=position_jitter(w=0.2,h=0), shape=".") +
  stat_summary(fun.y="mean", colour="red", geom="line") +
  stat_summary(fun.y="median", colour="blue", geom="line") +
  coord_cartesian(xlim=c(1950,2016), ylim=c(0,1e6))

# FullBaths vs YearBuilt
ggplot(homes, aes(x=YearBuilt, y=FullBath)) + geom_point()

# add some jitter, transparancy and zoom in
ggplot(homes, aes(x=YearBuilt, y=FullBath)) + 
  geom_jitter(alpha=1/8) +
  coord_cartesian(xlim=c(1950,2016))

# now add a line for the mean number of full baths
ggplot(homes, aes(x=YearBuilt, y=FullBath)) + 
  geom_jitter(alpha=1/8) +
  coord_cartesian(xlim=c(1950,2016), ylim=c(0,6)) +
  stat_summary(fun.y="mean", color="red", geom="line", size=2) +
  scale_y_continuous(breaks=0:6) +
  scale_x_continuous(breaks=seq(1960,2010,10))


# can also add summary stats manually
# first calculate the summary stats and put in data frame
avHome <- data.frame(x=median(homes$FinSqFt),
                     y=median(homes$TotalValue))
avHome # the average home

# make scatterplot and add summary stat;
# Notice we used two different data frames: homes and avHome
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point() +
  geom_point(data=avHome, aes(x=x,y=y), color="red", size=4)


# Multiple plots in one window --------------------------------------------


# In base R, we usually use par(mfrow=c(i,j)), like so:
par(mfrow=c(1,2))
hist(homes$FinSqFt)
plot(FinSqFt ~ YearBuilt, data=homes)
par(mfrow=c(1,1))

# We cannot use this approach for ggplot2. The easiest solution is to use the 
# grid.arrange() function in the gridExtra package. To use it, you have to save
# your plots and then call them using grid.arrange().

p1 <- ggplot(homes, aes(x=FinSqFt)) + geom_histogram() 
p2 <- ggplot(homes, aes(x=YearBuilt, y=FinSqFt)) + geom_point()
library(gridExtra)
grid.arrange(p1, p2, nrow=1) # can also use ncol



# Time-Permitting Miscellaneous Topics ------------------------------------


# adding uncertainty to graphs 

# Example: add means and CIs to strip chart 
# Data: chickwts (An experiment was conducted to measure and compare the
# effectiveness of various feed supplements on the growth rate of chickens.)
str(chickwts)

sc <- ggplot(chickwts, aes(x=feed, y=weight)) + 
  geom_jitter(width = 0.1, height = 0)
sc

# now to calculate means and standard errors
chick2 <- chickwts %>% 
  group_by(feed) %>% 
  summarise(fMean = mean(weight),
            fSE = sd(weight)/sqrt(length(weight)))

# now add mean and error bars
sc + geom_point(data=chick2, aes(x=feed, y=fMean), color="red", size=3) +
  geom_errorbar(data=chick2, aes(x=feed, y=fMean,
                                 ymin=fMean - 2*fSE, 
                                 ymax=fMean + 2*fSE), 
                width=0.1, color="red") +
  labs(title="Mean Weight by Feed Type with 2*SE error bars")

# another way using stat_summary; fun.data="mean_cl_normal" actually calls the 
# function smean.cl.normal() from the Hmisc package. It uses the t distribution
# to determine the multiplier of the standard error.
ggplot(chickwts, aes(x=feed, y=weight)) + 
  geom_point(position = position_jitter(w = 0.1, h = 0)) +
  stat_summary(fun.data = "mean_cl_normal", color="red", geom="errorbar", width=0.1) +
  stat_summary(fun.y = "mean", geom="point", color="red", size=3) +
  labs(title="Mean Weight by Feed Type with 95% error bars")



# single line graph of means at each time point with SE bars

# Indometh data (comes with R)
# Six subjects were given an intravenous injection of indomethacin at 11 times,
# and each time plasma concentrations of indometacin was measured.

# Here we make a line plot for each subject:
ggplot(Indometh, aes(x=time,y=conc, group=Subject)) +
  geom_line()

# Let's say we wanted to create a single line graph of means at each time point
# with SE bars.

# first calculate means and SEs
Indo2 <- Indometh %>% 
  group_by(time) %>% 
  summarise(tMean = mean(conc),
            tSE = sd(conc)/sqrt(length(conc)))

# now ready to create plot
ggplot(Indo2, aes(x=time,y=tMean)) +
  geom_line() +
  geom_errorbar(aes(ymin=tMean-2*tSE, ymax=tMean+2*tSE), width=0.1)

# or again we can use stat_summary()
ggplot(Indometh, aes(x=time,y=conc)) +
  stat_summary(fun.data="mean_cl_normal", geom="errorbar", width=0.1) +
  stat_summary(fun.y = "mean", geom = "line")


# maps
# ggplot can plot maps

# The following data are occurences of cougars in the US. It was downloaded from
# Biodiversity Information Serving Our Nation (BISON),
# http://bison.usgs.ornl.gov.

url <- "http://people.virginia.edu/~jcf2d/workshops/ggplot2/bison-Cougar-20150520-172801.csv"
cougar <- read.csv(url)

# install.packages("maps")
# install.packages("mapproj")

library(maps)
library(mapproj) # for the "polyconic" option

# Let's plot a map that shows occurences of courgars in the US and the type of occurence.

# first create a data frame of map data; "state" is the name of a map provided
# by the maps package.
states <- map_data("state")
# rename providedState to "region" and make lower case for purposes of merging
cougar$region <- tolower(cougar$providedState)
# merge states and courgar data frames by region
cougmap <- merge(states, subset(cougar, !is.na(decimalLongitude)), by = "region")

# now we're ready to plot! Map longitude and latitude to x and y aesthetics;
# The borders function draws the state borders;
# the coord_map("polyconic") function makes the map look curvy, like a globe;
# Give it a second...
ggplot(cougmap, aes(x=long, y=lat)) +
  borders("state") +
  geom_point(aes(x = decimalLongitude, y=decimalLatitude, color=basisOfRecord)) +
  coord_map("polyconic")

# saving graphs as images

# use ggsave(). It saves the last plot according to your file extension.
ggsave("cougars.jpg", width=10, height=5)


# Themes

# Using built-in themes such as theme_bw()
# Before
ggplot(airquality, aes(x=Temp)) + geom_histogram(binwidth=2) +
  facet_wrap(~Month)
# After
ggplot(airquality, aes(x=Temp)) + geom_histogram(binwidth=2) +
  facet_wrap(~Month) + theme_bw()
# another: theme_light
ggplot(airquality, aes(x=Temp)) + geom_histogram(binwidth=2) +
  facet_wrap(~Month) + theme_light()

# To permanently change the theme:
prevTheme <- theme_set(theme_bw())

ggplot(airquality, aes(x=Temp)) + geom_histogram(binwidth=2) +
  facet_wrap(~Month)

ggplot(airquality, aes(x=Temp)) + geom_freqpoly(binwidth=2) +
  facet_wrap(~Month)

# To restore
theme_set(prevTheme)
# verify
ggplot(airquality, aes(x=Temp)) + geom_freqpoly(binwidth=2) +
  facet_wrap(~Month)

# see package ggthemes for more themes!
# install.packages("ggthemes")
library(ggthemes)

# Wall Street Journal theme
ggplot(iris, aes(x=Petal.Length, y = Petal.Width, color = Species)) + 
  geom_point() + geom_smooth(se=F) + theme_wsj()

# google docs theme
ggplot(iris, aes(x=Petal.Length, y = Petal.Width, color = Species)) + 
  geom_point() + geom_smooth(se=F) + theme_gdocs()

# fivethirtyeight.com theme
ggplot(iris, aes(x=Petal.Length, y = Petal.Width, color = Species)) + 
  geom_point() + geom_smooth(se=F) + theme_fivethirtyeight()

# Tufte's theme (based on The Visual Display of Quantitative Information)
ggplot(iris, aes(x=Petal.Length, y = Petal.Width, color = Species)) + 
  geom_point() + geom_smooth(se=F) + theme_tufte()

# base R theme
ggplot(iris, aes(x=Petal.Length, y = Petal.Width, color = Species)) + 
  geom_point() + geom_smooth(se=F) + theme_base()
